;
(function() {
	_log("DfaceJSBridge load start");
	if (window.DfaceJSBridge) {
		_log("DfaceJSBridge has loaded");
		return;
	}

	var messageHandlers = {};
	var responseCallbacks = {};
	var uniqueId = 1;

	function init(messageHandler) {
		if (DfaceJSBridge._messageHandler) {
			throw new Error('init called twice');
		}
		DfaceJSBridge._messageHandler = messageHandler;
	}

	function send(data, responseCallback) {
		_doSend({
			data: data
		}, responseCallback);
	}

	function registerHandler(handlerName, handler) {
	    _log("registerHandler() -" + " handlerName:" + handlerName + "; handler:" + handler);
		messageHandlers[handlerName] = handler;
	}

	function callHandler(handlerName, data, responseCallback) {
	    _log("callHandler() -" + " handlerName:" + handlerName + "; data:" + data);
		_doSend({
			handlerName: handlerName,
			data: data
		}, responseCallback);
	}

	function _doSend(message, responseCallback) {
		if (responseCallback) {
			var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
			responseCallbacks[callbackId] = responseCallback;
			message['callbackId'] = callbackId;
		}
		if (message.data instanceof Object) {
		    message.data = JSON.stringify(message.data);
		}
		_WebViewJavaScriptBridge.handleMessageFromJs(message.data || null, message.responseId || null,
			message.responseData || null, message.callbackId || null, message.handlerName || null);
	}

	function _dispatchMessageFromJava(messageJSON) {
		var message = JSON.parse(messageJSON);
		var messageHandler;

		if (message.responseId) {
			var responseCallback = responseCallbacks[message.responseId];
			if (!responseCallback) {
				return;
			}
			responseCallback(message.responseData);
			delete responseCallbacks[message.responseId];
		} else {
			var responseCallback;
			if (message.callbackId) {
				var callbackResponseId = message.callbackId;
				responseCallback = function(responseData) {
					_doSend({
						responseId: callbackResponseId,
						responseData: responseData
					});
				}
			}

			var handler = DfaceJSBridge._messageHandler;
			if (message.handlerName) {
				handler = messageHandlers[message.handlerName];
			}
			try {
				handler(message.data, responseCallback);
			} catch (exception) {
				if (typeof console != 'undefined') {
					_log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
				}
			}
		}
	}

	function _handleMessageFromJava(messageJSON) {
		_dispatchMessageFromJava(messageJSON);
	}

	function _log(msg) {
	    console.log("JS - " + msg);
	}

	getDfaceSharedObject = function() {
        if (typeof(DfaceSharedObject) != 'undefined') {
            return JSON.stringify(DfaceSharedObject);
        }

        var getSharedImages = function() {
            if (typeof(DfaceSharedObject) != 'undefined') {
                return DfaceSharedObject.image;
            }

            var defaultimg = "http://static.dface.cn/img/share.png";
            var allImgs = document.getElementsByTagName("img");

            if (allImgs.length != 0) {
                for (var i = 0; i < allImgs.length; i++) {
                    var img = allImgs[i];
                    if (img.style.display == 'none' || img.style.visibility == 'hidden') {
                        continue;
                    }

                    if (img.width >= 100 || img.height >= 100) {
                        return img.src;
                    }
                }
            }

            return defaultimg;
        };

        var getSharedTitle = function() {
            if (typeof(DfaceSharedObject) != 'undefined') {
                return DfaceSharedObject.title;
            }

            var title = document.title;
            if (title == "") {
                title = '';
            }

            return title;
        };

        var getSharedDescription = function() {
            if (typeof(DfaceSharedObject) != 'undefined') {
                return DfaceSharedObject.descprition;
            }

            var title = document.title;
            if (title == "") {
                title = '';
            }

            return title;
        };

        var getSharedUrl = function() {
            if (typeof(DfaceSharedObject) != 'undefined') {
                return DfaceSharedObject.url;
            }

            return location.href;
        };

        var dfaceShareObject = {
            thumb: getSharedImages(),
            title: getSharedTitle(),
            desc: getSharedDescription(),
            url: getSharedUrl()
        };

        return JSON.stringify(dfaceShareObject);
    };

    getWxSharedObject = function() {
        if (typeof(wxSharedObject) != 'undefined') {
            return JSON.stringify(wxSharedObject);
        }

        var getSharedImages = function() {
            if (typeof(wxSharedObject) != 'undefined') {
                return wxSharedObject.image;
            }

            var defaultimg = "http://static.dface.cn/img/share.png";
            var allImgs = document.getElementsByTagName("img");

            if (allImgs.length != 0) {
                for (var i = 0; i < allImgs.length; i++) {
                    var img = allImgs[i];
                    if (img.style.display == 'none' || img.style.visibility == 'hidden') {
                        continue;
                    }

                    if (img.width >= 100 || img.height >= 100) {
                        return img.src;
                    }
                }
            }

            return defaultimg;
        };

        var getSharedTitle = function() {
            if (typeof(wxSharedObject) != 'undefined') {
                return wxSharedObject.title;
            }

            var title = document.title;
            if (title == "") {
                title = '';
            }

            return title;
        };

        var getSharedDescription = function() {
            if (typeof(wxSharedObject) != 'undefined') {
                return wxSharedObject.descprition;
            }

            var title = document.title;
            if (title == "") {
                title = '';
            }

            return title;
        };

        var getSharedUrl = function() {
            if (typeof(wxSharedObject) != 'undefined') {
                return wxSharedObject.url;
            }

            return location.href;
        };

        var WxSharedObject = {
            thumb: getSharedImages(),
            title: getSharedTitle(),
            desc: getSharedDescription(),
            url: getSharedUrl()
        };

        return JSON.stringify(WxSharedObject);
    };

	window.DfaceJSBridge = {
		init: init,
		send: send,
		registerHandler: registerHandler,
		callHandler: callHandler,
		call: callHandler,
		_handleMessageFromJava: _handleMessageFromJava
	};

	DfaceJSBridge.registerHandler("getDfaceSharedObject", function(data, responseCallback) {
            responseCallback(getDfaceSharedObject());
        });

	DfaceJSBridge.registerHandler("getWxSharedObject", function(data, responseCallback) {
    	    responseCallback(getWxSharedObject());
    	});

	var doc = document;
	var readyEvent = doc.createEvent('Events');
	readyEvent.initEvent('DfaceJSBridgeReady');
	readyEvent.bridge = DfaceJSBridge;
	doc.dispatchEvent(readyEvent);
	_log("DfaceJSBridge load done");
})();